<?php

/**
 * The admin-specific functionality of the plugin.
 *
 * @link       http://www.visual4.de/
 * @since      1.0.0
 *
 * @package    v4_wp1crm_Extensions
 * @subpackage v4_wp1crm_Extensions/admin
 */

/**
 * The admin-specific functionality of the plugin.
 *
 * Defines the plugin name, version, and two examples hooks for how to
 * enqueue the admin-specific stylesheet and JavaScript.
 *
 * @package    v4_wp1crm_Extensions
 * @subpackage v4_wp1crm_Extensions/admin
 * @author     visual4 GmbH <info@visual4.de>
 */
class v4_wp1crm_Extensions_Admin
{

    /**
     * The ID of this plugin.
     *
     * @since    1.0.0
     * @access   private
     * @var      string $plugin_name The ID of this plugin.
     */
    private $plugin_name;

    /**
     * The version of this plugin.
     *
     * @since    1.0.0
     * @access   private
     * @var      string $version The current version of this plugin.
     */
    private $version;

    /**
     * The display handler for additional fields.
     *
     * @since    1.0.0
     * @access   private
     * @var      string $version The current version of this plugin.
     */
    private $displayhandler_additionalfields;

    /**
     * Initialize the class and set its properties.
     *
     * @since    1.0.0
     * @param      string $plugin_name The name of this plugin.
     * @param      string $version The version of this plugin.
     */
    public function __construct($plugin_name, $version)
    {

        $this->load_dependencies();

        $this->plugin_name = $plugin_name;
        $this->version = $version;

        $this->displayhandler_additionalfields = new displayhandler_additionalfields();

		add_filter('woocommerce_billing_fields', [$this, 'wc_edit_billing_fields'], 10, 2);
		add_filter('woocommerce_address_to_edit', [$this, 'wc_edit_address_fields'], 10, 2);

	}

    private function load_dependencies() {

        require_once 'partials/v4-wp1crm-extensions-admin-displayhandler-additionalfields.php';

    }

    /**
     * Register the stylesheets for the admin area.
     *
     * @since    1.0.0
     */
    public function enqueue_styles()
    {

        /**
         * This function is provided for demonstration purposes only.
         *
         * An instance of this class should be passed to the run() function
         * defined in v4_wp1crm_Extensions_Loader as all of the hooks are defined
         * in that particular class.
         *
         * The v4_wp1crm_Extensions_Loader will then create the relationship
         * between the defined hooks and the functions defined in this
         * class.
         */

        wp_enqueue_style($this->plugin_name, plugin_dir_url(__FILE__) . 'css/v4-wp1crm-extensions-admin.css', array(), $this->version, 'all');

    }

    /**
     * Register the JavaScript for the admin area.
     *
     * @since    1.0.0
     */
    public function enqueue_scripts()
    {

        /**
         * This function is provided for demonstration purposes only.
         *
         * An instance of this class should be passed to the run() function
         * defined in v4_wp1crm_Extensions_Loader as all of the hooks are defined
         * in that particular class.
         *
         * The v4_wp1crm_Extensions_Loader will then create the relationship
         * between the defined hooks and the functions defined in this
         * class.
         */

        wp_enqueue_script($this->plugin_name, plugin_dir_url(__FILE__) . 'js/v4-wp1crm-extensions-admin.js', array('jquery'), $this->version, false);

    }

    /**
     * Include Endpointhandler Class
     */
    public function include_endpointhandler_class()
    {
        include_once('class-v4-wp1crm-extensions-endpointhandler.php');
    }

    public function add_endpointhandler_class_to_wc($classes)
    {
        $classes[] = 'Class_v4_wp1crm_Extensions_Endpointhandler';
        return $classes;
    }

    /**
     * @param bool $async
     * @param WC_Webhook $WC_Webhook
     * @param $arg
     * @return bool
     */
    public function woowebhook_deliver_async_only_on_frontend($async, $WC_Webhook, $arg)
    {
        $synchronous = get_option('woocommerce_api_synchronous') == 'yes' ? true : false;
        if (is_admin() && $synchronous) {
            $async = false;
        }

        return $async;
    }

    public function woocommerce_max_webhook_delivery_failures()
    {
        return 999999;
    }

    public function add_admin_notice($message, $is_error = false)
    {
        $msg_exists = false;
        if (isset($_SESSION['wpwh_error']))
            foreach ($_SESSION['wpwh_error'] as $error)
                if($message == $error['msg'])
                    $msg_exists = true;

        if(!$msg_exists)
            $_SESSION['wpwh_error'][] = [
                'msg' => $message,
                'is_error' => $is_error
            ];
    }

    public function display_admin_error()
    {
        if (isset($_SESSION['wpwh_error'])) {
            foreach ($_SESSION['wpwh_error'] as $error) {
                ?>
                <div class="notice <?php if ($error['is_error']) echo 'notice-error'; ?>">
                    <p><?= $error['msg'] ?></p>
                </div>
                <?php
            }
            unset($_SESSION['wpwh_error']);
        }
    }

    /**
     * @param array $http_args
     * @param array|WP_Error $response
     * @param $duration
     * @param $arg
     * @param $wc_webhook_id
     */
    public function admin_notice_on_delivery_failure($http_args, $response, $duration, $arg, $wc_webhook_id)
    {
        $synchronous = get_option('woocommerce_api_synchronous') == 'yes' ? true : false;
        if($synchronous){
            if (is_a($response, 'WP_ERROR')) {
                $this->add_admin_notice($response->get_error_message(), true);
            } else if (is_array($response) && $response['response']['code'] >= 300) {
                $this->add_admin_notice($response['body'], true);
            } else {
				if(isset($_SESSION['wpwh_error']))
                	unset($_SESSION['wpwh_error']);
            }
        }
    }

    /**
     * send mail if webhook fails
     */
    public function add_errorhandling_option_to_wc_settings($settings)
    {
        $settings[] = array(
            'title' => 'WC-API E-Mail Benachrichtigung',
            'desc' => 'sendet eine E-Mail an den Administrator, wenn der Webhook fehlschlägt.',
            'id' => 'woocommerce_api_send_error',
            'type' => 'checkbox',
            'default' => 'no',
        );
        $settings[] = array(
            'title' => 'WC-API Backend synchron zustellen',
            'desc' => 'Webhooks synchron zustellen (nur Backend). Wartet auf Antwort und zeigt Fehler an (verlangsamt den Speichervorgang).',
            'id' => 'woocommerce_api_synchronous',
            'type' => 'checkbox',
            'default' => 'no',
        );
        return $settings;
    }

    public function send_mail_if_webhook_fails($http_args, $response)
    {
        $send_error = get_option('woocommerce_api_send_error') == 'yes';
        if ($send_error) {
        	if (is_a($response, 'WP_Error')) {
				$res_msg = $response->get_error_message();
				$res_code = $response->get_error_code();
			} else {
				$res_msg = $response['response']['message'];
				$res_code = $response['response']['code'];
			}
			if($res_code == 200)
				return;

			/** @var WP_Error $response */
			$admin_email = get_option('admin_email');
			$home_url = rtrim(get_home_url(), "/");
			$req_hook = $http_args['headers']['X-WC-Webhook-Topic'];
			$req_body = $http_args['body'];
			$body = json_decode($req_body, true);
			$order_id = $body['order']['id'] ?? false;
			if ($order_id && $home_url)
				$link = "$home_url/wp-admin/post.php?post=$order_id&action=edit";
			else
				$link = false;
			$message =
				"Error: " . $res_code
				. "\nMessage: $res_msg"
				. "\nWebhook: $req_hook";
//				. "\nRequest: $req_body";
			if ($link) $message .= "\nOrder Link: $link";
//            $message .= "\n\n!IMPORTANT! Webhook will be deactivated after a few errors!";
			mail($admin_email, "WooCommerce Webhook Error <$home_url>", $message);
        }
    }

    public function catch_contact_crm_id($http_args, $response)
    {
        if (!is_a($response, 'WP_Error')) {
            $response_code = !empty($response['response']['code']) ? $response['response']['code'] : false;
            if($response_code == 200){
				$result = json_decode($response['body'], true);
				if(!empty($result['Contact'][0]['id']) && count($result['Contact']) === 1){
					if($http_args['headers']['X-WC-Webhook-Resource'] == 'customer') {
						$request = json_decode($http_args['body'], true);
						$user_id = $request['id'];
						$user = get_user_by('id', $user_id);
						if (is_a($user, 'WP_User'))
						    update_user_meta($user_id, 'crm_id', $result['Contact'][0]['id']);
					}
				}
			}
        }
    }

    public function checkout_custom_fields_validation($data, $errors)
    {
        $wp1crm = get_option('wp1crm');
        if($wp1crm['b2b'] && $data['billing_country'] != 'DE' && empty($data['tax_information']))
            wc_add_notice(sprintf(__('%s is a required field.', 'woocommerce'), '<strong>' . esc_html(__('VAT ID', 'v4-wp1crm-extensions')) . '</strong>'), 'error');
    }

    public function add_cost_field_to_woocommerce_product_options_pricing()
    {
        /** @var WC_Product $product_object */
        global $product_object;

        woocommerce_wp_text_input( array(
            'id'        => '_cost',
//            'value'     => wc_format_decimal($product_object->get_meta('cost', true)),
            'value'     => wc_format_decimal($product_object->get_meta('cost', true)),
            'label'     => __( 'Cost price', 'woocommerce' ) . ' (' . get_woocommerce_currency_symbol() . ')',
            'data_type' => 'price',
        ) );
    }

    /**
     * @param $loop
     * @param $variation_data
     * @param WP_Post $variation
     */
    public function add_cost_field_to_woocommerce_variation_options_pricing($loop, $variation_data, $variation)
    {
        $product = wc_get_product($variation->ID);
        $cost = $product->get_meta('cost', true);
        $val = wc_format_decimal($cost);

        woocommerce_wp_text_input( array(
            'id'            => "variable_cost_{$loop}",
            'name'          => "variable_cost[{$loop}]",
            'value'         => $val,
//            'value'         => '',
            'label'         => __( 'Cost price', 'woocommerce' ) . ' (' . get_woocommerce_currency_symbol() . ')',
            'data_type'     => 'price',
            'wrapper_class' => 'form-row form-row-first',
        ) );
    }

    /**
     * before save
     *
     * @param WC_Product $product
     */
    public function set_cost_to_woocommerce_product_meta($product)
    {
        $raw_price = wc_clean($_POST['_cost']);
        $raw_price = wc_format_decimal($raw_price);
        $raw_price = (float) $raw_price;
        $product->add_meta_data('cost', $raw_price, true);
    }

    /**
     * after save
     *
     * @param $variation_id
     * @param $i
     */
    public function set_cost_to_woocommerce_variation_meta($variation_id, $i)
    {
        $product = wc_get_product($variation_id);
        $raw_price = wc_clean( $_POST['variable_cost'][$i]);
        $raw_price = wc_format_decimal($raw_price);
        $raw_price = (float) $raw_price;
        $product->add_meta_data('cost', $raw_price, true);

        $product->save();
    }

    /**
     * add raw prices to order webhook (Legacy v3)
     *
     * @deprecated
     *
     * @param $order_data
     * @param $order WC_Order
     * @return mixed
     */
    public function add_data_to_order_webhook($order_data, $order)
    {
        //add data to order_data
        $order_data['total_raw'] = (double) $order->get_total();
        $order_data['subtotal_raw'] = (double) $order->get_subtotal();
        $order_data['total_tax_raw'] = (double) $order->get_total_tax();
        $order_data['total_shipping_raw'] = (double) $order->get_shipping_total();
        $order_data['cart_tax_raw'] = (double) $order->get_cart_tax();
        $order_data['shipping_tax_raw'] = (double) $order->get_shipping_tax();
        $order_data['total_discount_raw'] = (double) $order->get_total_discount();

        //add data to line items
        $items = $order->get_items();
        for ($i = 0; $i < count($order_data['line_items']); $i++) {
            $line_item = &$order_data['line_items'][$i];
            $item = $items[$line_item['id']];

            $line_item['subtotal_raw'] = (double) $order->get_line_subtotal($item, false, false);
            $line_item['subtotal_tax_raw'] = (double) $item['line_subtotal_tax'];
            $line_item['total_raw'] = (double) $order->get_line_total($item, false, false);
            $line_item['total_tax_raw'] = (double) $item['line_tax'];
            $line_item['price_raw'] = (double) $order->get_item_total($item, false, false);

			$product = wc_get_product($item->get_product_id());
            if(is_a($product, 'WC_Product')) {
				$line_item['cost'] = (double)$product->get_meta('cost');
				$line_item['category_ids'] = $product->get_category_ids();
			}
        }

        //add data to shipping lines
        $shipping_items = $order->get_shipping_methods();
        for($i = 0; $i < count($order_data['shipping_lines']); $i++){
            $shipping_line = &$order_data['shipping_lines'][$i];
            $shipping_item = $shipping_items[$shipping_line['id']];

            $shipping_line['total_raw'] = (double) $shipping_item['cost'];
        }

        //add data to tax lines
        $taxes = $order->get_tax_totals();
        for($i = 0; $i < count($order_data['tax_lines']); $i++){
            $tax_line = &$order_data['tax_lines'][$i];
            $tax = $taxes[$tax_line['code']];

            $tax_line['total_raw'] = (double) $tax->amount;
        }

        //add data to fee lines
        $fee_items = $order->get_fees();
        for($i = 0; $i < count($order_data['fee_lines']); $i++){
            $fee_line = &$order_data['fee_lines'][$i];
            $fee_item = $fee_items[$fee_line['id']];

            $fee_line['total_raw'] = (double) $order->get_line_total( $fee_item, false, false );
            $fee_line['total_tax_raw'] = (double) $fee_item['line_tax'];
        }

        //add data to coupon lines
        $coupon_items = $order->get_items( 'coupon' );
        for($i = 0; $i < count($order_data['coupon_lines']); $i++){
            $coupon_line = &$order_data['coupon_lines'][$i];
            $coupon_item = $coupon_items[$coupon_line['id']];

            $coupon_line['amount_raw'] = (double) $coupon_item['discount_amount'];
        }

        return $order_data;
    }


    /**
     * add data to shop order rest response (WP API v2)
     *
     * @param WP_REST_Response $response The response object.
     * @param WC_Order         $object   Object data.
     * @param WP_REST_Request  $request  Request object.
     * @return WP_REST_Response
     */
    public function add_data_to_shop_order_rest_response($response, $object, $request)
    {
        $cstm_order_data = &$response->data;

        //add data to order_data
        $cstm_order_data['total_raw'] = (double) $object->get_total();
        $cstm_order_data['subtotal_raw'] = (double) $object->get_subtotal();
        $cstm_order_data['total_tax_raw'] = (double) $object->get_total_tax();
        $cstm_order_data['total_shipping_raw'] = (double) $object->get_shipping_total();
        $cstm_order_data['cart_tax_raw'] = (double) $object->get_cart_tax();
        $cstm_order_data['shipping_tax_raw'] = (double) $object->get_shipping_tax();
        $cstm_order_data['total_discount_raw'] = (double) $object->get_total_discount();

        //add data to line items
        $items = $object->get_items();
        for ($i = 0; $i < count($cstm_order_data['line_items']); $i++) {
            $line_item = &$cstm_order_data['line_items'][$i];
            $item = $items[$line_item['id']];

            $line_item['subtotal_raw'] = (double) $object->get_line_subtotal($item, false, false);
            $line_item['subtotal_tax_raw'] = (double) $item['line_subtotal_tax'];
            $line_item['total_raw'] = (double) $object->get_line_total($item, false, false);
            $line_item['total_tax_raw'] = (double) $item['line_tax'];
            $line_item['price_raw'] = (double) $object->get_item_total($item, false, false);

            $product = wc_get_product($item->get_product_id());
            if(is_a($product, 'WC_Product')) {
				$line_item['cost'] = (double)$product->get_meta('cost');
				$line_item['category_ids'] = $product->get_category_ids();
			}
        }

        //add data to shipping lines
        $shipping_items = $object->get_shipping_methods();
        for($i = 0; $i < count($cstm_order_data['shipping_lines']); $i++){
            $shipping_line = &$cstm_order_data['shipping_lines'][$i];
            $shipping_item = $shipping_items[$shipping_line['id']];

            $shipping_line['total_raw'] = (double) $shipping_item['cost'];
        }

        //add data to tax lines
        $taxes = $object->get_tax_totals();
        for($i = 0; $i < count($cstm_order_data['tax_lines']); $i++){
            $tax_line = &$cstm_order_data['tax_lines'][$i];
            $tax = $taxes[$tax_line['rate_code']];

            $tax_line['total_raw'] = (double) $tax->amount;
        }

        //add data to fee lines
        $fee_items = $object->get_fees();
        for($i = 0; $i < count($cstm_order_data['fee_lines']); $i++){
            $fee_line = &$cstm_order_data['fee_lines'][$i];
            $fee_item = $fee_items[$fee_line['id']];

            $fee_line['total_raw'] = (double) $object->get_line_total( $fee_item, false, false );
            $fee_line['total_tax_raw'] = (double) $fee_item['line_tax'];
        }

        //add data to coupon lines
        $coupon_items = $object->get_items( 'coupon' );
        if(!empty($cstm_order_data['coupon_lines']))
            for($i = 0; $i < count($cstm_order_data['coupon_lines']); $i++){
                $coupon_line = &$cstm_order_data['coupon_lines'][$i];
                $coupon_item = $coupon_items[$coupon_line['id']];

                $coupon_line['amount_raw'] = (double) $coupon_item['discount_amount'];
            }

        return $response;
    }

    /**
     * Add menues for the admin area.
     *
     * @since    1.0.0
     */
    public function add_menue()
    {
        // create new top-level menu
        add_menu_page('WP 1CRM Extensions', 'WP 1CRM Extensions', 'manage_options', 'wp1crm-extensions', array($this, 'display_pluginOverview'));
        add_submenu_page('wp1crm-extensions', 'Overview', 'Overview', 'manage_options', 'wp1crm-extensions', array($this, 'display_pluginOverview'));
        add_submenu_page('wp1crm-extensions', 'Additional Fields Settings', 'v4 Woo Custom Fields', 'manage_options', 'wp1crm-additional-fields', array($this, 'display_additionalFields'));
        add_submenu_page('wp1crm-extensions', 'v4 Tools', 'v4 Tools', 'manage_options', 'wp1crm-tools', array($this, 'display_tools'));

    }

    function register_wp1crmadditionalfields_plugin_settings()
    {
        //register our settings
        register_setting(
            'wp1crmadditionalfields-group', // Option group
            'wp1crm', // Option name
            array($this, 'sanitize_settings') // Sanitize
        );

        register_setting('wp1crmadditionalfields-group', v4_wp1crm_Extensions_Public_Setting::$SECTIONS['billing']['htmlid']);
        register_setting('wp1crmadditionalfields-group', v4_wp1crm_Extensions_Public_Setting::$SECTIONS['shipping']['htmlid']);
        register_setting('wp1crmadditionalfields-group', v4_wp1crm_Extensions_Public_Setting::$SECTIONS['account']['htmlid']);
        register_setting('wp1crmadditionalfields-group', v4_wp1crm_Extensions_Public_Setting::$SECTIONS['order']['htmlid']);

    }

    function sanitize_settings($input){
        //checkboxes
        $input['b2b'] = (isset($input['b2b']) && $input['b2b'] == '1');
        $input['company_readonly'] = (isset($input['company_readonly']) && $input['company_readonly'] == '1');
        $input['email_readonly'] = (isset($input['email_readonly']) && $input['email_readonly'] == '1');
//        $input['invoice_as'] = (isset($input['invoice_as']) && $input['invoice_as'] == '1');

        return $input;
    }

	function wc_edit_billing_fields($address_fields, $country)
	{
		if(is_user_logged_in()){
			$wp1crm = get_option('wp1crm');
			if($wp1crm['company_readonly']){
				$address_fields['billing_company']['custom_attributes']['readonly'] = 'readonly';
			}
			if($wp1crm['email_readonly']){
				$address_fields['billing_email']['custom_attributes']['readonly'] = 'readonly';
			}
		}
		return $address_fields;
	}

	function wc_edit_address_fields($address, $load_address){
		if($load_address == 'billing'){
			$wp1crm = get_option('wp1crm');
			if($wp1crm['company_readonly']){
				$address['billing_company']['custom_attributes']['readonly'] = 'readonly';
			}
			if($wp1crm['email_readonly']){
				$address['billing_email']['custom_attributes']['readonly'] = 'readonly';
			}
		}
		return $address;
	}

    function display_pluginOverview(){
        $this->displayhandler_additionalfields->overview();
    }

    function display_additionalFields()
    {
        $this->displayhandler_additionalfields->additional_fields();
    }

	function display_tools()
	{
		$this->displayhandler_additionalfields->tools();
	}

    function frm_api_request_args($arg_array, $args){
        $arg_array['headers']['x-wc-webhook-source'] = home_url('/');
        $arg_array['headers']['x-wc-webhook-resource'] = 'service_case';

		//signature
		$wpwh_options = get_option('v4_wpwh');
		$secret = $wpwh_options['secret'];
		$hash_raw = hash_hmac('sha256', $arg_array['body'], $secret, true);
		$hash = base64_encode($hash_raw);
		$arg_array['headers']['x-wc-webhook-signature'] = $hash;

        return $arg_array;
    }

    function woocommerce_before_product_object_save($obj, $data_store)
    {
		if (is_object($obj)) {
			$sku = $obj->get_sku();
			if(empty($sku))
				add_filter( 'redirect_post_location', array( $this, 'add_missing_sku_notice_query_var' ), 99 );

			if (is_a('WC_Product_Variable', $obj)) {
				//TODO check children
			}
        }
    }

	public function add_missing_sku_notice_query_var($location)
	{
		remove_filter('add_missing_sku_notice_query_var', array($this, 'add_notice_query_var'), 99);
		return add_query_arg(array('wp1crm_admin_notice' => 'missing_sku'), $location);
	}

    function admin_notices()
    {
		if (!isset($_GET['wp1crm_admin_notice']))
			return;
			
		switch ($_GET['wp1crm_admin_notice']){
            case 'missing_sku':
                $msg = __('Bitte geben Sie eine Artikelnummer ein', 'v4-wp1crm-extensions');
                break;
        }
		if(empty($class)) $class = 'error notice';
		if(empty($msg)) $msg = __('Es ist ein Fehler bei der Synchronisation zu 1CRM aufgetreten', 'v4-wp1crm-extensions');
		
        ?>
        <div class="<?= $class ?>">
            <p><?= $msg ?></p>
        </div>
        <?php
    }

    function woocommerce_product_options_stock()
    {
        //action "woocommerce_product_options_stock"
        global $pagenow;
        if (is_admin() && !in_array($pagenow, array('post-new.php'))) { //check for new post page
            ?>
            <div class="stock_fields">
                <p class="form-field _sku_field ">
                    <label for="_sku"><abbr title="Artikelnummer">1CRM Info</abbr></label>
                    <span>Änderungen an der "Lagerbestandsmenge" bitte im 1CRM verwalten.</span>
                </p>
                <script>
                    $(document).ready(function () {
                        $('#_stock').attr('disabled', 'disabled');
                    });
                </script>
            </div>
            <?php
        }
    }

	function woocommerce_billing_fields($address_fields)
	{
	    if(!empty($address_fields['billing_title'])){
            $address_fields['billing_title']['options'][3] = 'Divers';
        }
		unset($address_fields['billing_state']);
		return $address_fields;
	}

	function woocommerce_shipping_fields($address_fields)
	{
	    if(!empty($address_fields['shipping_title'])){
            $address_fields['shipping_title']['options'][3] = 'Divers';
        }
		unset($address_fields['shipping_state']);
		return $address_fields;
	}

	/**
	 * @param WC_Customer $obj
	 * @param $data_store
	 */
	public function add_cookie_consent_to_customer($obj, $data_store)
	{
	    if(is_a($obj, 'WC_Customer')){
			$v4core_cookie_setting_necessary = (isset($_COOKIE['v4core_cookie_setting_necessary']) && $_COOKIE['v4core_cookie_setting_necessary'] == 1);
			if(!is_admin() && $v4core_cookie_setting_necessary){
				if(isset($_COOKIE['v4core_cookie_setting_user_id']) && wp_is_uuid($_COOKIE['v4core_cookie_setting_user_id']))
					update_user_meta($obj->get_id(), 'v4core_cookie_setting_user_id', $_COOKIE['v4core_cookie_setting_user_id']);
				update_user_meta($obj->get_id(), 'v4core_cookie_setting_necessary', $v4core_cookie_setting_necessary);
				update_user_meta($obj->get_id(), 'v4core_cookie_setting_preferences', (isset($_COOKIE['v4core_cookie_setting_preferences']) && $_COOKIE['v4core_cookie_setting_preferences'] == 1));
				update_user_meta($obj->get_id(), 'v4core_cookie_setting_statistics', (isset($_COOKIE['v4core_cookie_setting_statistics']) && $_COOKIE['v4core_cookie_setting_statistics'] == 1));
				update_user_meta($obj->get_id(), 'v4core_cookie_setting_marketing', (isset($_COOKIE['v4core_cookie_setting_marketing']) && $_COOKIE['v4core_cookie_setting_marketing'] == 1));
			}
        }
	}

	function formidable_send_cookie_settings($arg_array, $args)
	{
		$v4core_cookie_setting_necessary = (isset($_COOKIE['v4core_cookie_setting_necessary']) && $_COOKIE['v4core_cookie_setting_necessary'] == 1);
		if(!is_admin() && $v4core_cookie_setting_necessary && isset($arg_array['body'])){
			$json_obj = json_decode($arg_array['body']);

			if(isset($_COOKIE['v4core_cookie_setting_user_id']) && wp_is_uuid($_COOKIE['v4core_cookie_setting_user_id']))
				$json_obj->cookie_settings['user_id'] = $_COOKIE['v4core_cookie_setting_user_id'];
			$json_obj->cookie_settings['is_necessary_set'] = $v4core_cookie_setting_necessary;
			$json_obj->cookie_settings['is_preferences_set'] = (isset($_COOKIE['v4core_cookie_setting_preferences']) && $_COOKIE['v4core_cookie_setting_preferences'] == 1);
			$json_obj->cookie_settings['is_statistics_set'] = (isset($_COOKIE['v4core_cookie_setting_statistics']) && $_COOKIE['v4core_cookie_setting_statistics'] == 1);
			$json_obj->cookie_settings['is_marketing_set'] = (isset($_COOKIE['v4core_cookie_setting_marketing']) && $_COOKIE['v4core_cookie_setting_marketing'] == 1);

			$arg_array['body'] = json_encode($json_obj);
		}

		return $arg_array;
	}

	public function add_additional_roles_to_customer($response, $user_data, $request)
	{
		$user = new WP_User($user_data->ID);
		$response_customer_data = $response->get_data();

		$response_customer_data['additional_roles'] = $user->roles;

		$response->set_data($response_customer_data);
		return $response;
	}

	public function extra_user_profile_fields($user)
	{
		if (!current_user_can('edit_users', $user->ID)) {
			return false;
		}
		?>
        <h3><?php _e("1CRM", "blank"); ?></h3>

        <table class="form-table">
            <tr>
                <th><label for="crm_id"><?php _e("1CRM Kontakt ID"); ?></label></th>
                <td>
                    <input type="text" name="crm_id" id="crm_id"
                           value="<?php echo esc_attr(get_user_meta($user->ID, 'crm_id', true)); ?>"
                           class="regular-text"/><br/>
                </td>
            </tr>
        </table>
		<?php
	}

	public function save_extra_user_profile_fields($user_id)
	{
		if (empty($_POST['_wpnonce']) || !wp_verify_nonce($_POST['_wpnonce'], 'update-user_' . $user_id)) {
			return false;
		}
		if (!current_user_can('edit_users', $user_id)) {
			return false;
		}
		update_user_meta($user_id, 'crm_id', $_POST['crm_id']);
		return true;
	}

}
