<?php

/**
 * The file that defines the core plugin class
 *
 * A class definition that includes attributes and functions used across both the
 * public-facing side of the site and the admin area.
 *
 * @link       http://www.visual4.de/
 * @since      1.0.0
 *
 * @package    v4_wp1crm_Extensions
 * @subpackage v4_wp1crm_Extensions/includes
 */

/**
 * The core plugin class.
 *
 * This is used to define internationalization, admin-specific hooks, and
 * public-facing site hooks.
 *
 * Also maintains the unique identifier of this plugin as well as the current
 * version of the plugin.
 *
 * @since      1.0.0
 * @package    v4_wp1crm_Extensions
 * @subpackage v4_wp1crm_Extensions/includes
 * @author     visual4 GmbH <info@visual4.de>
 */
class v4_wp1crm_Extensions {

    /**
	 * The loader that's responsible for maintaining and registering all hooks that power
	 * the plugin.
	 *
	 * @since    1.0.0
	 * @access   protected
	 * @var      v4_wp1crm_Extensions_Loader    $loader    Maintains and registers all hooks for the plugin.
	 */
	protected $loader;

	/**
	 * @var v4_wp1crm_Extensions_Updater $updater
	 */
	protected $updater;

    /**
	 * The unique identifier of this plugin.
	 *
	 * @since    1.0.0
	 * @access   protected
	 * @var      string    $plugin_name    The string used to uniquely identify this plugin.
	 */
	protected $plugin_name;

    /**
     * The current version of the plugin.
     *
     * @since    1.0.0
     * @access   protected
     * @var      string    $version    The current version of the plugin.
     */
    protected $version;

    /**
     * The Webhookhandler Class.
     *
     * @since    1.0.0
     * @access   protected
     * @var      Class_v4_wp1crm_Extensions_Webhookhandler    $webhookhandler
     */
    protected $webhookhandler;

    /**
     * The oAuth Class.
     *
     * @since    1.0.0
     * @access   protected
     * @var      Class_v4_wp1crm_Extensions_oAuth    $oauth
     */
    protected $oauth;

    /**
     * The EM_WC Class.
     *
     * @since    1.0.0
     * @access   protected
     * @var      Class_v4_wp1crm_Extensions_EM_WC    $em_wc
     */
    protected $em_wc;

    /**
     * The EM_WC_WPML Class.
     *
     * @since    1.0.0
     * @access   protected
     * @var      Class_v4_wp1crm_Extensions_EM_WC_WPML    $em_wc_wpml
     */
    protected $em_wc_wpml;

    /**
	 * Define the core functionality of the plugin.
	 *
	 * Set the plugin name and the plugin version that can be used throughout the plugin.
	 * Load the dependencies, define the locale, and set the hooks for the admin area and
	 * the public-facing side of the site.
	 *
	 * @since    1.0.0
	 */
	public function __construct() {

		$this->plugin_name = 'v4-wp1crm-extensions';
		$this->version = '1.0.1';

		$this->load_dependencies();
		$this->set_locale();
		$this->define_admin_hooks();
		$this->define_public_hooks();

	}

	/**
	 * Load the required dependencies for this plugin.
	 *
	 * Include the following files that make up the plugin:
	 *
	 * - v4_wp1crm_Extensions_Loader. Orchestrates the hooks of the plugin.
	 * - v4_wp1crm_Extensions_i18n. Defines internationalization functionality.
	 * - v4_wp1crm_Extensions_Admin. Defines all hooks for the admin area.
	 * - v4_wp1crm_Extensions_Public. Defines all hooks for the public side of the site.
	 *
	 * Create an instance of the loader which will be used to register the hooks
	 * with WordPress.
	 *
	 * @since    1.0.0
	 * @access   private
	 */
	private function load_dependencies() {

		/**
		 * The class responsible for orchestrating the actions and filters of the
		 * core plugin.
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/class-v4-wp1crm-extensions-loader.php';

		/**
		 * The class responsible for defining internationalization functionality
		 * of the plugin.
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/class-v4-wp1crm-extensions-i18n.php';

        /**
         * The class responsible for handling all webhooks.
         */
        require_once plugin_dir_path( dirname( __FILE__ ) ) . 'admin/class-v4-wp1crm-extensions-webhookhandler.php';

        $this->webhookhandler = new Class_v4_wp1crm_Extensions_Webhookhandler();

        /**
         * The class responsible for oauth extensions.
         */
        require_once plugin_dir_path( dirname( __FILE__ ) ) . 'admin/class-v4-wp1crm-extensions-oauth.php';

        $this->oauth = new Class_v4_wp1crm_Extensions_oAuth();

        /**
         * The class responsible for em_wc extensions.
         */
        require_once plugin_dir_path( dirname( __FILE__ ) ) . 'admin/class-v4-wp1crm-extensions-em-wc.php';
        require_once plugin_dir_path( dirname( __FILE__ ) ) . 'admin/class-v4-wp1crm-extensions-em-wc-wpml.php';
        $this->em_wc = new Class_v4_wp1crm_Extensions_EM_WC();
        $this->em_wc_wpml = new Class_v4_wp1crm_Extensions_EM_WC_WPML();

		/**
		 * The class responsible for defining all actions that occur in the admin area.
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'admin/class-v4-wp1crm-extensions-admin.php';

		/**
		 * The class responsible for defining all actions that occur in the public-facing
		 * side of the site.
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'public/class-v4-wp1crm-extensions-public.php';

        /**
         * Additional Fields
         */
        require_once plugin_dir_path( dirname( __FILE__ ) ) . 'public/class-v4-wp1crm-extensions-public-helper.php';
        require_once plugin_dir_path( dirname( __FILE__ ) ) . 'public/class-v4-wp1crm-extensions-public-setting.php';

		/**
		 * v4_wp1crm_Extensions_Updater
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/class-v4-wp1crm-extensions-updater.php';
		$this->updater = new v4_wp1crm_Extensions_Updater();


        $this->loader = new v4_wp1crm_Extensions_Loader();

	}

	/**
	 * Define the locale for this plugin for internationalization.
	 *
	 * Uses the v4_wp1crm_Extensions_i18n class in order to set the domain and to register the hook
	 * with WordPress.
	 *
	 * @since    1.0.0
	 * @access   private
	 */
	private function set_locale() {

		$plugin_i18n = new v4_wp1crm_Extensions_i18n();

		$this->loader->add_action( 'plugins_loaded', $plugin_i18n, 'load_plugin_textdomain' );

	}

	/**
	 * Register all of the hooks related to the admin area functionality
	 * of the plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 */
	private function define_admin_hooks() {

		$plugin_admin = new v4_wp1crm_Extensions_Admin( $this->get_plugin_name(), $this->get_version() );

		$this->loader->add_action( 'admin_enqueue_scripts', $plugin_admin, 'enqueue_styles' );
		$this->loader->add_action( 'admin_enqueue_scripts', $plugin_admin, 'enqueue_scripts' );

		// add Endpoint
        $this->loader->add_action('woocommerce_api_loaded', $plugin_admin, 'include_endpointhandler_class');
        $this->loader->add_filter('woocommerce_api_classes', $plugin_admin, 'add_endpointhandler_class_to_wc');

        // set woocommerce_max_webhook_delivery_failures very high
        $this->loader->add_filter('woocommerce_max_webhook_delivery_failures', $plugin_admin, 'woocommerce_max_webhook_delivery_failures');
        // admin notice on failure
        $this->loader->add_action('admin_notices', $plugin_admin, 'display_admin_error');
        $this->loader->add_filter('woocommerce_webhook_deliver_async', $plugin_admin, 'woowebhook_deliver_async_only_on_frontend', 10, 3); //fires product webhook three times. WooCommerce default.
        $this->loader->add_action('woocommerce_webhook_delivery', $plugin_admin, 'admin_notice_on_delivery_failure', 10, 5);

        // webhook response send error message via mail to admin
        $this->loader->add_filter('woocommerce_settings_features', $plugin_admin, 'add_errorhandling_option_to_wc_settings');
        $this->loader->add_action('woocommerce_webhook_delivery', $plugin_admin, 'send_mail_if_webhook_fails', 10, 2);

        // save crm_id in current user
        $this->loader->add_action('woocommerce_webhook_delivery', $plugin_admin, 'catch_contact_crm_id', 10, 2);

        // additional fields
        $this->loader->add_action('admin_menu', $plugin_admin, 'add_menue' );
        $this->loader->add_action('admin_init', $plugin_admin, 'register_wp1crmadditionalfields_plugin_settings' );

        $this->loader->add_action('woocommerce_after_checkout_validation', $plugin_admin, 'checkout_custom_fields_validation', 10, 2 );

        // add cost to product
        $this->loader->add_action('woocommerce_product_options_pricing', $plugin_admin, 'add_cost_field_to_woocommerce_product_options_pricing' );
        $this->loader->add_action('woocommerce_variation_options_pricing', $plugin_admin, 'add_cost_field_to_woocommerce_variation_options_pricing', 10, 3 );
        $this->loader->add_action('woocommerce_admin_process_product_object', $plugin_admin, 'set_cost_to_woocommerce_product_meta' );
        $this->loader->add_action('woocommerce_save_product_variation', $plugin_admin, 'set_cost_to_woocommerce_variation_meta', 10, 2 );

        // formidable. prio 999 because of signature
        $this->loader->add_filter( 'frm_api_request_args', $plugin_admin, 'frm_api_request_args', 999, 2 );

        /** Legacy API */
        // add prices with all decimals to order
        $this->loader->add_filter( 'woocommerce_api_order_response', $plugin_admin, 'add_data_to_order_webhook', 10, 2);
        /** WP API */
        // add custom data to order to order
//        $this->loader->add_filter( "woocommerce_rest_prepare_{$post_type}_object", $plugin_admin, 'add_data_to_{$post_type}_rest_response', 10, 4);
        $this->loader->add_filter( "woocommerce_rest_prepare_shop_order_object", $plugin_admin, 'add_data_to_shop_order_rest_response', 10, 3);

        $this->loader->add_filter( 'woocommerce_rest_prepare_customer', $plugin_admin, 'add_additional_roles_to_customer', 10, 3 );

		$this->loader->add_filter( "woocommerce_after_data_object_save", $plugin_admin, 'add_cookie_consent_to_customer', 10, 2);
		$this->loader->add_filter( "woocommerce_after_customer_object_save", $plugin_admin, 'add_cookie_consent_to_customer', 10, 2);

		//make SKU required
        $this->loader->add_action('woocommerce_before_product_object_save', $plugin_admin, 'woocommerce_before_product_object_save', 10, 2);
        $this->loader->add_action('admin_notices', $plugin_admin, 'admin_notices');

        //disable stock changes on existing products
        $this->loader->add_action( "woocommerce_product_options_stock", $plugin_admin, 'woocommerce_product_options_stock', 10, 0);

		//salutation other
		$this->loader->add_filter('woocommerce_billing_fields', $plugin_admin, 'woocommerce_billing_fields');
		$this->loader->add_filter('woocommerce_shipping_fields', $plugin_admin, 'woocommerce_shipping_fields');

		//formidable cookie settings
		$this->loader->add_filter('frm_api_request_args', $plugin_admin, 'formidable_send_cookie_settings', 20, 2);

		// additional user fields
		$this->loader->add_action('show_user_profile', $plugin_admin, 'extra_user_profile_fields');
		$this->loader->add_action('edit_user_profile', $plugin_admin, 'extra_user_profile_fields');
		$this->loader->add_action('personal_options_update', $plugin_admin, 'save_extra_user_profile_fields');
		$this->loader->add_action('edit_user_profile_update', $plugin_admin, 'save_extra_user_profile_fields');
    }

	/**
	 * Register all of the hooks related to the public-facing functionality
	 * of the plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 */
	private function define_public_hooks() {

		$plugin_public = new v4_wp1crm_Extensions_Public( $this->get_plugin_name(), $this->get_version() );

		$this->loader->add_action( 'wp_enqueue_scripts', $plugin_public, 'enqueue_styles' );
		$this->loader->add_action( 'wp_enqueue_scripts', $plugin_public, 'enqueue_scripts' );

		// additional fields
        $this->loader->add_action('woocommerce_webhook_http_args', $plugin_public, 'add_http_args_to_woocommerce_webhook');
        $this->loader->add_action('woocommerce_checkout_update_order_meta', $plugin_public, 'custom_order_field_update' );
        $this->loader->add_action('woocommerce_checkout_fields', $plugin_public, 'display_custom_fields_in_checkout' );

        //add "my Bookings" to "my Account" UPDATE: is now an em-woo default function
//        $this->loader->add_action('woocommerce_account_bookings_endpoint', $plugin_public, 'emp_bookings_endpoint_content');
//        $this->loader->add_action('init', $plugin_public, 'emp_add_my_account_endpoint');
//        $this->loader->add_filter('woocommerce_account_menu_items', $plugin_public, 'emp_account_menu_items');
//        $this->loader->add_filter('the_title', $plugin_public, 'emp_custom_endpoint_title');

        //validation
//        $this->loader->add_action('woocommerce_after_checkout_validation', $plugin_public, 'wc_checkout_check_email', 10, 2);

    }

	/**
	 * Run the loader to execute all of the hooks with WordPress.
	 *
	 * @since    1.0.0
	 */
	public function run() {
		$this->loader->run();
	}

	/**
	 * The name of the plugin used to uniquely identify it within the context of
	 * WordPress and to define internationalization functionality.
	 *
	 * @since     1.0.0
	 * @return    string    The name of the plugin.
	 */
	public function get_plugin_name() {
		return $this->plugin_name;
	}

	/**
	 * The reference to the class that orchestrates the hooks with the plugin.
	 *
	 * @since     1.0.0
	 * @return    v4_wp1crm_Extensions_Loader    Orchestrates the hooks of the plugin.
	 */
	public function get_loader() {
		return $this->loader;
	}

	/**
	 * Retrieve the version number of the plugin.
	 *
	 * @since     1.0.0
	 * @return    string    The version number of the plugin.
	 */
	public function get_version() {
		return $this->version;
	}

}
